package adt.dto;

import adt.entity.Host;
import adt.entity.Port;
import adt.entity.ScanInfo;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;

import javax.persistence.Id;
import javax.persistence.Transient;
import java.util.Date;
import java.util.List;
import java.net.NetworkInterface;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.net.SocketException;

@Data
@NoArgsConstructor
@AllArgsConstructor
public class ScanInfoDto {

    @Id
    @JsonProperty("vlan_id")
    String vlanName;

    @JsonProperty("node_ip_address")
    String nodeIp;

    @JsonProperty("timestamp_dist_node_scan_start")
    @JsonFormat(pattern="yyyy-MM-dd HH:mm:ss")
    private Date iniTimestampScan;

    @JsonProperty("timestamp_dist_node_scan_end")
    @JsonFormat(pattern="yyyy-MM-dd HH:mm:ss")
    private Date endTimestampScan;

    @JsonProperty("mac_address")
    private String macAddress;

    @JsonProperty("devices")
    private List<Host> host;

    public static ScanInfoDto createScanInfoDto(String vlanName, String ip, Date iniTimestampScan, Date endTimestampScan) {

        ScanInfoDto scanInfo = new ScanInfoDto();
        scanInfo.setVlanName(vlanName);
        scanInfo.setNodeIp(ip);
        scanInfo.setIniTimestampScan(iniTimestampScan);
        scanInfo.setEndTimestampScan(endTimestampScan);

        String ipAddress = ip.toString();
        byte[] mac = null;
        try {
            InetAddress add = InetAddress.getByName(ipAddress);
            NetworkInterface network = NetworkInterface.getByInetAddress(add);
            mac = network.getHardwareAddress();
        }catch(UnknownHostException ue){
            // Handled
        }catch(SocketException se){
            // Handled
        }
        scanInfo.setMacAddress(mac.toString());
        //scanInfo.setHosts(new ArrayList<>());

        return scanInfo;
    }

}
