package adt.entity;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonFormat;
import lombok.Data;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.Transient;
import java.util.Date;
import java.util.List;
import com.fasterxml.jackson.annotation.JsonIgnore;
import javax.persistence.*;

@Data
@Entity
public class Host {

    @Id
    @JsonProperty("ip_address")
    private String ip;

    @JsonProperty("hostname")
    private String hostname;

    @ManyToOne
    @JsonProperty("scan_info")
    private ScanInfo scanInfo;

    @JsonProperty("mac_address")
    private String mac;

    @JsonProperty("mac_manufacturer")
    private String macManufacturer;

    @JsonProperty("timestamp_device_scan_start")
    @JsonFormat(pattern="yyyy-MM-dd HH:mm:ss")
    private Date timestamp_ini;

    @JsonProperty("timestamp_device_scan_end")
    @JsonFormat(pattern="yyyy-MM-dd HH:mm:ss")
    private Date timestamp_end;

    @Transient
    @JsonProperty("ports")
    private List<Port> ports;

    @JsonProperty("operating_system")
    private String operatingSystem;

    @JsonIgnore
    @JsonProperty("vlan_id")
    private String vlanName;

    @JsonProperty("discovery_method")
    private String discoveryMethod;

    public static Host createHost(String ip, String hostname, String mac, String discoveryMethod) {
        Host host = new Host();
        host.setIp(ip);
        host.setHostname(hostname);
        host.setMac(mac);
        host.setMacManufacturer("");
        host.setDiscoveryMethod(discoveryMethod);
        return host;
    }

    public static Host createHost(String ip, String discoveryMethod) {
        Host host = new Host();
        host.setIp(ip);
        host.setMacManufacturer("");
        host.setDiscoveryMethod(discoveryMethod);
        return host;
    }

    public static Host createHost(String ip, String hostname, String mac, String vlanName, Date timestampIni, ScanInfo scanInfo, String discoveryMethod) {
        Host host = new Host();
        host.setIp(ip);
        host.setHostname(hostname);
        host.setMac(mac);
        host.setScanInfo(scanInfo);
        host.setMacManufacturer("");
        host.setVlanName(vlanName);
        host.setTimestamp_ini(timestampIni);
        host.setDiscoveryMethod(discoveryMethod);
        return host;
    }

}

