package adt.entity;

import adt.entity.Host;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonFormat;
import lombok.Getter;
import lombok.Setter;
import lombok.NoArgsConstructor;
import com.fasterxml.jackson.annotation.JsonIgnore;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.Transient;
import java.util.Date;
import java.util.List;
import java.util.ArrayList;
import java.net.NetworkInterface;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.net.SocketException;

import lombok.extern.slf4j.Slf4j;
import javax.persistence.*;


@Slf4j
@Entity
@Getter
@Setter
@NoArgsConstructor
public class ScanInfo {

    @Id
    @JsonProperty("vlan_id")
    String vlanName;

    @JsonProperty("node_ip_address")
    String nodeIp;

    @JsonProperty("timestamp_dist_node_scan_start")
    @JsonFormat(pattern="yyyy-MM-dd HH:mm:ss")
    private Date iniTimestampScan;

    @JsonProperty("timestamp_dist_node_scan_end")
    @JsonFormat(pattern="yyyy-MM-dd HH:mm:ss")
    private Date endTimestampScan;

    @JsonProperty("mac_address")
    private String macAddress;

    @Transient
    @JsonProperty("devices")
    private List<Host> host;

    @JsonProperty("discovery_method")
    private String discoveryMethod;

    public static ScanInfo createScanInfo(String vlanName, String ip, Date iniTimestampScan, String discoveryMethod) {

        ScanInfo scanInfo = new ScanInfo();
        scanInfo.setVlanName(vlanName);
        scanInfo.setNodeIp(ip);
        scanInfo.setIniTimestampScan(iniTimestampScan);

        scanInfo.setDiscoveryMethod(discoveryMethod);

        String ipAddress = ip.toString();
        String auxMac = null;
        byte[] mac = null;
        try {

            InetAddress add = InetAddress.getByName(ipAddress);

            NetworkInterface network = NetworkInterface.getByInetAddress(add);

            if(network != null) {
                mac = network.getHardwareAddress();
            }else{
                auxMac = "unknown";
            }
        }catch(UnknownHostException ue){
            // Handled
        }catch(SocketException se){
            // Handled
        }

        if(mac != null) {

            StringBuilder sb = new StringBuilder();

            for (int i = 0; i < mac.length; i++) {
                sb.append(String.format("%02X%s", mac[i],
                        (i < mac.length - 1) ? "-" : ""));
            }

            scanInfo.setMacAddress(sb.toString());

        }else{

            scanInfo.setMacAddress(auxMac);
        }

        return scanInfo;
    }

    public static ScanInfo createScanInfo(String vlanName, String ip, Date iniTimestampScan, Date endTimestampScan, String discoveryMethod) {

        ScanInfo scanInfo = new ScanInfo();
        scanInfo.setVlanName(vlanName);
        scanInfo.setNodeIp(ip);
        scanInfo.setIniTimestampScan(iniTimestampScan);
        scanInfo.setEndTimestampScan(endTimestampScan);

        scanInfo.setDiscoveryMethod(discoveryMethod);

        String ipAddress = ip.toString();
        String auxMac = null;
        byte[] mac = null;
        try {

            InetAddress add = InetAddress.getByName(ipAddress);

            NetworkInterface network = NetworkInterface.getByInetAddress(add);

            if(network != null) {
                mac = network.getHardwareAddress();
            }else{
                auxMac = "unknown";
            }
        }catch(UnknownHostException ue){
            // Handled
        }catch(SocketException se){
            // Handled
        }

        if(mac != null) {

            StringBuilder sb = new StringBuilder();

            for (int i = 0; i < mac.length; i++) {
                sb.append(String.format("%02X%s", mac[i],
                        (i < mac.length - 1) ? "-" : ""));
            }

            scanInfo.setMacAddress(sb.toString());

        }else{

            scanInfo.setMacAddress(auxMac);
        }

        return scanInfo;
    }

}

