package adt.newportdetection.runner;

import adt.dto.Asset;
import adt.entity.Host;
import adt.entity.Port;
import adt.entity.ScanInfo;
import com.google.common.net.InetAddresses;
import javafx.util.Pair;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.util.*;
import java.util.concurrent.Semaphore;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Stream;

@Slf4j
public class ProcessWindows extends Thread{
    private static int N = 4;
    private static double width_window = 0.5;
    private static int porcentaje = 75;
    private static ArrayList<Map<Pair<String,Integer>,PortDetection>> windows;
    private static double [] tiempos_mayores;
    private static BufferedReader buffer;
    private static double tiempo_limite_superior;
    private static int ventana_actual;
    private static int ventana_primera;
    private static Pattern pattern;
    private static Map<String,String> localhosts;
    private static Map<String,Map<String,String>> vlans;
    private static ListPortDetection listPortDetection;
    private static Map<String, String> name_vlans;
    private final Semaphore mutex_windows  = new Semaphore(1);
    private final Semaphore mutex_buffer = new Semaphore(1);
    private final SendInfoToMain sendInfoToMain;
    private static boolean terminar = false;

    @Value("${token}")
    private String token;

    /* ====================================
     *   Constructor
     *  ====================================
     * */

    public ProcessWindows(BufferedReader buffer, ListPortDetection listPortDetection, SendInfoToMain sendInfoToMain, List<String[]> vlanIps) {
        this.sendInfoToMain = sendInfoToMain;
        ProcessWindows.tiempos_mayores = new double[N];
        ProcessWindows.windows = new ArrayList<>(N);
        for(int i = 0; i < N; i++){
            ProcessWindows.windows.add(i,new HashMap<>());
        }
        ProcessWindows.buffer = buffer;
        ProcessWindows.listPortDetection = listPortDetection;
        ProcessWindows.pattern = Pattern.compile("([0-9]+:[0-9]+:[0-9]+.[0-9]+)\\sIP\\s([0-9]+.[0-9]+.[0-9]+.[0-9]+).([0-9]+) > ([0-9]+.[0-9]+.[0-9]+.[0-9]+).([0-9]+): .* length ([0-9]+)");
        ProcessWindows.tiempo_limite_superior = -1;
        ProcessWindows.ventana_actual = -1;
        ProcessWindows.ventana_primera = 0;
        for (String[] vlanIp : vlanIps) {
            name_vlans.put(vlanIp[0],vlanIp[1]);
            Map<String, String> aux;
            if (ProcessWindows.localhosts == null) {
                aux = new HashMap<>();
            } else {
                aux = ProcessWindows.localhosts;
            }
            List<String> aux_list = getIpHost(vlanIp[0], Integer.parseInt(vlanIp[2]));
            Map<String,String> map_ip_ip = new HashMap<>();
            for (String s : aux_list) {
                aux.put(s, s);
                map_ip_ip.put(s, s);
            }
            ProcessWindows.vlans.put(vlanIp[0],map_ip_ip);
            ProcessWindows.localhosts = aux;
        }
    }

    /* ====================================
     *   Metodo que obtiene las ips de las diferentes maquinas
     *  ====================================
     * */

    private List<String> getIpHost(String subnetIPAddress, int subnetMask) {
        List<String> resultado = new ArrayList<>();
        long limit = (long) (Math.pow(2, 32 - subnetMask) - 1);
        InetAddress firstAddress = InetAddresses.forString(subnetIPAddress);
        Stream<InetAddress> stream = Stream.iterate(firstAddress, InetAddresses::increment).limit(limit);
        stream.forEach(addr -> {
            try {
                if (addr.isReachable(5000)){
                    resultado.add(addr.getHostAddress());
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        });
        return resultado;
    }

    /* ====================================
    *   Metodo que lee una linea del buffer
    *  ====================================
    * */

    private String readLine(){
        String line ="";
        try {
            mutex_buffer.acquire();
            line = buffer.readLine();
            mutex_buffer.release();
        } catch (IOException | InterruptedException e) {
//            e.printStackTrace();
        }
        return line;
    }

    /* ====================================
     *   Metodo Principal del algoritmo el cual llama a todas las funciones
     *   intermedias del Algoritmo de Ventanas
     *  ====================================
     * */

    private void addWindows(PortDetection portDetection) throws InterruptedException {
        // Inicializacion
        double time = this.getSeconds(portDetection);
        if (tiempo_limite_superior == -1){
            // Inicializar tiempo_limite_superior
            tiempo_limite_superior =  time + (N * width_window);
        }
        if (ventana_actual == -1){
            // inicializar Ventana y vector de tiempos_mayores
            ventana_actual = 0;
            for (int i = 0; i< tiempos_mayores.length; i++)
            {
                tiempos_mayores[i] = (time + width_window * (i + 1)) - 0.000001;
            }
        }
        // Algoritmo
        if (tiempo_limite_superior < time) {
            //Se ha salido de todas las ventana
            this.desbordamiento(portDetection);
        }else {
            if (tiempos_mayores[ventana_actual] < time){
                // Pasa a meterse en la ventana siguiente
                if (ventana_actual == N-1){
                    ventana_actual = 0;
                }else {
                    ventana_actual++;
                }
            }
            mutex_windows.acquire();
            windows.get(ventana_actual).put(new Pair<>(portDetection.getIp(),portDetection.getPort()),portDetection);
            mutex_windows.release();
        }
    }
    /* ====================================
     *   Metodo que se encarga de realizar las accciones
     *   cuando llega un paquete que no esta en las actuales
     *   ventanas y resolver este problema
     *  ====================================
     * */
    private void desbordamiento(PortDetection portDetection) throws InterruptedException {
        int count = 0;
        int i = ventana_primera;
        Set<Pair<String,Integer>> claves_primera = windows.get(ventana_primera).keySet();
        for (Pair<String, Integer> clave : claves_primera) {
            int contador = 0;
            while (count < N-1) {
                if (i == N) i = 0;
                Set<Pair<String, Integer>> claves = windows.get(i).keySet();
                if (claves.contains(clave)) {
                    contador++;
                }
                count++;
                i++;
            }
            if ((contador * 100 / N ) >= porcentaje){
                // Si puerto supera el porcentaje, se añade a la lista de PortDetection
                listPortDetection.addNewPort(windows.get(ventana_primera).get(clave));
//                log.info("=====================================================");
//                log.info("windows: " + windows.toString());
//                log.info("ventana_primera: " + ventana_primera + "");
//                log.info("ventana_actual: " + ventana_actual + "");
//                log.info("tiempos_mayores: " + Arrays.toString(tiempos_mayores));
//                log.info("tiempo_limite_superior: " + tiempo_limite_superior +"");
                log.info(listPortDetection.toString());
//                log.info("===========================
//                 ==========================");
            }
//            log.info("=====================================================");
//            log.info("windows: " + windows.toString());
//            log.info("ventana_primera: " + ventana_primera + "");
//            log.info("ventana_actual: " + ventana_actual + "");
//            log.info("tiempos_mayores: " + Arrays.toString(tiempos_mayores));
//            log.info("tiempo_limite_superior: " + tiempo_limite_superior +"");
//            log.info(listPortDetection.toString());
//            log.info("=====================================================");
        }
        // Enviar la informacion al nodo Main
        sendAsset();
        // Vaciar la ventana_primera
        windows.remove(ventana_primera);
        windows.add(ventana_primera,new HashMap<>());
        if (ventana_primera==0) {
            tiempos_mayores[ventana_primera] = tiempos_mayores[N-1] + width_window;
        }else {
            tiempos_mayores[ventana_primera] = tiempos_mayores[ventana_primera - 1] + width_window;
        }
        if (ventana_primera == N-1){
            ventana_primera = 0;
        }else {
            ventana_primera++;
        }
        tiempo_limite_superior += width_window;
        if (ventana_actual == N -1){
            ventana_actual = 0;
        }else ventana_actual++;
        mutex_windows.acquire();
        windows.get(ventana_actual).put(new Pair<>(portDetection.getIp(),portDetection.getPort()),portDetection);
        mutex_windows.release();
    }
    /* ====================================
     *   Metodo que obtiene los segundos a partir del un objeto Portdetection
     *  ====================================
     * */
    private double getSeconds(PortDetection portDetection){
        String[] time = portDetection.getTime().split(":");
        return Integer.parseInt(time[0]) * 3600 + Integer.parseInt(time[1]) * 60 + Double.parseDouble(time[2]);
    }

    /* ====================================
     *   Metodo que realiza la comprobacion de si la linea leida del buffer
     *   es una linea que tiene las caracteristicas propias para ser añadida
     *   a las ventanas
     *  ====================================
     * */

    private ArrayList<PortDetection> match(String line) {
        Matcher matcher = pattern.matcher(line);
        ArrayList<PortDetection> resultado = new ArrayList<>();
        if (matcher.find()) {
            String time = matcher.group(1);
            String source = matcher.group(2);
            String destination = matcher.group(4);
            int portS = Integer.parseInt(matcher.group(3));
            int portD = Integer.parseInt(matcher.group(5));
            int length = Integer.parseInt(matcher.group(6));
            if (localhosts.containsKey(source)) {
                resultado.add(new PortDetection(time, source, portS, length));
            }
            if (localhosts.containsKey(destination)) {
                resultado.add(new PortDetection(time, destination, portD, length));
            }

        }
        return resultado;
    }

    /* ====================================
     *   Metodo que se llama cuando se inicia el Thread
     *  ====================================
     * */
    public void start() {
        try {
            String line = null;
            while ( (line = readLine()) != null && !terminar){
                ArrayList<PortDetection> meter = match(line);
                for (PortDetection portDetection : meter) {
                    addWindows(portDetection);
                }
            }
        }catch (InterruptedException e){
            e.printStackTrace();

        }

    }
    private void sendAsset(){
            Asset asset = new Asset();
            Map<String, ScanInfo> list_scan_info = new HashMap<>();
            for (Map.Entry<String, Map<Integer, Integer>> entry2 : listPortDetection.getIp_puertos().entrySet()) {
                    String vlan = "";
                    for (Map.Entry<String, Map<String, String>> entry  : vlans.entrySet()) {
                        if (entry.getValue().containsValue(entry2.getKey())) {
                            vlan = entry.getKey();
                            break;
                        }
                    }
                    Host host = Host.createHost(entry2.getKey(), "sniffing");
                    List<Port> ports = new ArrayList<>();
                    for (Map.Entry<Integer, Integer> entry1 : entry2.getValue().entrySet()) {
                        ports.add(Port.createPort("" + entry1.getKey(), host, "tcp", "sniffing"));
                    }
                    host.setPorts(ports);
                    if (list_scan_info.containsKey(vlan)){
                        // Ya esta esa vlan
                        list_scan_info.get(vlan).getHost().add(host);
                    }else {
                        // No esta esa vlan
                        ScanInfo aux = ScanInfo.createScanInfo(vlan,entry2.getKey(),null,"sniffing");
                        List<Host> meter_host = new ArrayList<>();
                        meter_host.add(host);
                        aux.setHost(meter_host);
                        list_scan_info.put(vlan,aux);
                    }
                }
            asset.setScanInfo(new ArrayList<>(list_scan_info.values()));
            asset.setDiscoveryMethod("sniffing");
            sendInfoToMain.sendInfo(asset, token);
            listPortDetection = new ListPortDetection();
    }


   /* ====================================
    * GETTERS AND SETTERS
    * ====================================
    * */
    public static int getN() {
        return N;
    }

    public static void setN(int n) {
        N = n;
    }

    public static double getWidth_window() {
        return width_window;
    }

    public static void setWidth_window(double width_window) {
        ProcessWindows.width_window = width_window;
    }

    public static int getPorcentaje() {
        return porcentaje;
    }

    public static void setPorcentaje(int porcentaje) {
        ProcessWindows.porcentaje = porcentaje;
    }
}
