package adt.nmap.reader;

import lombok.extern.slf4j.Slf4j;
import net.bytebuddy.dynamic.scaffold.MethodGraph;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

@Slf4j
public class NmapXmlReaderImpl implements NmapXmlReader {

    private DocumentBuilderFactory factory;
    private Document doc;

    @Override
    public void openFile(String fileName) {
        factory.setIgnoringElementContentWhitespace(true);
        DocumentBuilder builder = getNewDocumentBuilder(factory);

        if (builder != null) {
            Document doc = getNewDocument(builder, fileName);
            if (doc != null) {
                doc.getDocumentElement().normalize();
                this.doc = doc;
            }
        }
    }

    private Document getNewDocument(DocumentBuilder builder, String fileName) {
        File file = new File(fileName);
        Document doc = null;

        try {
            doc = builder.parse(file);
        } catch (SAXException | IOException e) {
            log.error("XML file {} does not have a correct structure.", fileName);
        }

        return doc;
    }

    private DocumentBuilder getNewDocumentBuilder(DocumentBuilderFactory factory) {
        DocumentBuilder builder = null;

        try {
            builder = factory.newDocumentBuilder();
        } catch (ParserConfigurationException e) {
            e.printStackTrace();
        }

        return builder;
    }

    @Override
    public List<Element> getOpenPorts() {

        List<Element> elements = new ArrayList<>();
        NodeList elementsByTagName = doc.getElementsByTagName("port");

        for (int i = 0; i < elementsByTagName.getLength(); i++) {
            elements.add((Element) elementsByTagName.item(i));
        }

        return elements;

    }

    @Override
    public List<Element> getServiceOnPort() {

        List<Element> elements = new ArrayList<>();
        Node node = doc.getElementsByTagName("port").item(0);

        NodeList childNodes = null;

        if(node != null) {
            childNodes = node.getChildNodes();

            for (int i = 0; i < childNodes.getLength(); i++) {
                if ("state".equals(childNodes.item(i).getNodeName()) || "service".equals(childNodes.item(i).getNodeName()))
                    elements.add((Element) childNodes.item(i));
            }

            return elements;
        }

        return null;

    }


    @Override
    public Element/*List<Element>*/ getOperativeSystem() {

        //List<Element> elements = new ArrayList<>();
        Element element = null;
        NodeList elementsByTagName = doc.getElementsByTagName("osmatch");

        /*
        for (int i = 0; i < elementsByTagName.getLength(); i++) {

            elements.add((Element) elementsByTagName.item(i));

        }
        */

        element = (Element) elementsByTagName.item(0);

        return element;

    }

    public NmapXmlReaderImpl() {
        this.factory = DocumentBuilderFactory.newInstance();
    }

}
