package adt.nmap.runner;

import adt.nmap.reader.NmapXmlReader;
import adt.nmap.reader.NmapXmlReaderImpl;
import lombok.extern.slf4j.Slf4j;
import org.w3c.dom.Element;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

@Slf4j
public class NmapRunnerImpl implements NmapRunner {

    @Override
    public List<Element> scanPortsOnHost(String ip) {

        List<Element> result = new ArrayList<>();

        String fileName = "/tmp/" + String.format(PORTS_RESULTS_FILENAME_FORMAT, ip);

        final String command = String.format("nmap -sT -T5 -p- %s -oX %s", ip, fileName);
        Process process = getProcessFromCommand(command);

        if (process != null) {

            NmapXmlReader nmapXmlReader = new NmapXmlReaderImpl();
            nmapXmlReader.openFile(fileName);
            result =  nmapXmlReader.getOpenPorts();

        }

        return result;

    }

    private Process getProcessFromCommand(String command) {
        Process process = null;
        try {
            process = Runtime.getRuntime().exec(command);
            process.waitFor();
        } catch (IOException | InterruptedException e) {
            log.error("Command {} interrupted.", command);
        }

        return process;
    }


    @Override
    public List<Element> detectServiceRunningOnPort(String ip, String port) {
        // Also allows to get OS info (-A option)

        List<Element> result = new ArrayList<>();

        String fileName = "/tmp/" + String.format(SERVICE_RUNNING_FILENAME_FORMAT, ip, port);

        final String command = String.format("nmap -sV -O -T5 %s -p %s -oX %s", ip, port, fileName);

        Process process = getProcessFromCommand(command);
        log.info("Service discovery for IP {} || PORT {} successfully completed.", ip, port);
        if (process != null) {
            NmapXmlReader nmapXmlReader = new NmapXmlReaderImpl();
            nmapXmlReader.openFile(fileName);

            result = nmapXmlReader.getServiceOnPort();

            if(result != null) {

                Element os = nmapXmlReader.getOperativeSystem();
                if (os != null) {
                    result.add(os);
                }

            }
        }
        log.info("Finished iteration for IP {} || PORT {} \n\n", ip, port);
        return result;
    }


}
