package adt.scanner;

import adt.entity.Host;
import adt.entity.Port;
import adt.nmap.runner.NmapRunner;
import adt.nmap.runner.NmapRunnerImpl;
import lombok.extern.slf4j.Slf4j;
import org.w3c.dom.Element;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.concurrent.Executors;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.function.Consumer;

@Slf4j
public class PortScanner {

    private final List<Host> hosts;
    private Host host;
    private long timeout;
    private Consumer<Port> onPortFound;


    public List<Port> scan(Host host, String discMethod) {

        List<Port> portsList = new ArrayList<>();

        NmapRunner nmapRunner = new NmapRunnerImpl();
        List<Element> portsOpen = nmapRunner.scanPortsOnHost(host.getIp());

        log.info("Scanning {} ports for {}", portsOpen.size(), host.getIp());

        portsOpen.forEach(element -> {
            Port port = Port.createPort(element.getAttribute("portid"), host, element.getAttribute("protocol"), discMethod);

            log.info("Port {} found running on host {}", port.getPortNumber(), port.getHost().getIp());

            List<Element> service = nmapRunner.detectServiceRunningOnPort(port.getHost().getIp(), port.getPortNumber());

            if(service != null){

                service.forEach(serviceDetails -> {

                    switch (serviceDetails.getNodeName()) {
                        case "state":
                            port.setState(serviceDetails.getAttribute("state"));
                            break;
                        case "service":
                            port.setServiceRunning(serviceDetails.getAttribute("product"));
                            port.setServiceVersion(serviceDetails.getAttribute("version"));
                            port.setDiscoveryMethod(serviceDetails.getAttribute("method"));
                            break;
                        case "osmatch":
                            host.setOperatingSystem(serviceDetails.getAttribute("name"));
                            break;
                        default:
                            break;
                    }
                });

                portsList.add(port);
            }

        });

        Date endTimestamp_device = new Date();
        host.setTimestamp_end(endTimestamp_device);

        return portsList;
    }



    /**
     *
     * @param onPortFound Callback function to be called every time a port is found
     * @return this for chaining
     */
    public PortScanner setOnPortFound(Consumer<Port> onPortFound) {
        this.onPortFound = onPortFound;
        return this;
    }

    /**
     *
     * @param timeout Waiting time in SECONDS.
     * @return this for chaining
     */
    public PortScanner setTimeout(long timeout) {
        this.timeout = timeout;
        return this;
    }

    public static PortScanner fromHosts(List<Host> hosts) {
        return new PortScanner(hosts);
    }

    public static PortScanner fromUniqueHost(Host host) {
        return new PortScanner(host);
    }

    private PortScanner(List<Host> hosts) {
        this.hosts = hosts;
        this.onPortFound = System.out::println;
    }

    private PortScanner(Host host) {
        this.host = host;
        this.hosts = null;
        this.onPortFound = System.out::println;
    }

}
