package adt.service;

import adt.entity.Host;
import adt.entity.Port;
import adt.repository.HostRepository;
import adt.repository.PortRepository;
import adt.scanner.HostScanner;
import adt.scanner.MacScanner;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
@Slf4j
public class HostScannerService {

    private final HostRepository hostRepository;
    private final PortRepository portRepository;
    private final PortScannerService portScannerService;
    private final WrittingService writtingService;

    @Value("${vlan.address}")
    private String vlanAddress;
    @Value("${vlan.mask}")
    private String vlanMask;

    @Autowired
    public HostScannerService(HostRepository hostRepository, PortRepository portRepository, PortScannerService portScannerService, WrittingService writtingService) {
        this.hostRepository = hostRepository;
        this.portRepository = portRepository;
        this.portScannerService = portScannerService;
        this.writtingService = writtingService;
    }


    void scanNetwork(String vlanName, String netAddress, String mask) {

        HostScanner hostScanner = HostScanner.fromSubnetAddress(String.format("%s/%s", netAddress, mask/*vlanAddress, vlanMask*/), portScannerService);
                hostScanner.setTimeout(5000)
                .setOnHostFound(this::onHostFound)
                .scan();

        List<Host> hostsList = hostScanner.getHostsList();

        for(int i=0; i<hostsList.size(); i++){

            hostsList.get(i).setVlanName(vlanName);
        }

        hostRepository.saveAll(hostsList);

        log.info("Scanning process finished.");
    }


    private void onHostFound(Host host) {

        log.info("Host {} found", host.getIp());

        String macManufacturer = MacScanner.getMacScanner().getMacManufacturer(host);
        host.setMacManufacturer(macManufacturer);
        hostRepository.save(host);
    }


}
