package adt.service;

import adt.entity.Host;
import adt.entity.Port;
import adt.nmap.runner.NmapRunner;
import adt.nmap.runner.NmapRunnerImpl;
import adt.repository.PortRepository;
import adt.scanner.PortScanner;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.w3c.dom.Element;

import java.util.List;
import lombok.Getter;
import lombok.Setter;

@Service
@Slf4j
@Getter
@Setter
public class PortScannerService {

    private final PortRepository portRepository;

    private List<Port> portList;

    private List<Port> portListDHCP;

    @Value("${vlan.port_timeout}")
    private long timeout;

    @Autowired
    public PortScannerService(PortRepository portRepository, List<Port> portList, List<Port> portListDHCP) {
        this.portRepository = portRepository;
        this.portList = portList;
        this.portListDHCP = portListDHCP;
    }

    public void scanHost(Host host, String discMethod) {

        log.info("Starting port scanning from host {}", host.getIp());
        List<Port> aux = PortScanner.fromUniqueHost(host)
                .setOnPortFound(this::onPortFound)
                .setTimeout(timeout)
                .scan(host, discMethod);
        log.info("Port scanning from host {} successfully finished.", host.getIp());

        if( discMethod == "dhcp") {

            try {
                synchronized (portListDHCP) {
                    portListDHCP.addAll(aux);
                }
            }catch(Exception e){
                log.info("[RUNTIME ERROR --PORT SCANNER SERVICE--] --- {}", e);
            }

        }else if( discMethod == "scan") {

            synchronized (portList) {
                portList.addAll(aux);
            }

        }
    }


    private void onPortFound(Port port) {

        log.info("Port {} found running on host {}", port.getPortNumber(), port.getHost().getIp());

        NmapRunner nmapRunner = new NmapRunnerImpl();
        List<Element> service = nmapRunner.detectServiceRunningOnPort(port.getHost().getIp(), port.getPortNumber());

        service.forEach(element -> {
            switch (element.getNodeName()) {
                case "state":
                    port.setState(element.getAttribute("state"));
                case "service":
                    port.setServiceRunning(element.getAttribute("product"));
                    port.setServiceVersion(element.getAttribute("version"));
                    port.setDiscoveryMethod(element.getAttribute("method"));
            }
        });

        portRepository.save(port);
    }


}
