package adt.service;

import adt.client.MainNodeClient;
import adt.entity.Vlan;
import adt.dto.Node;
import adt.newportdetection.runner.ListPortDetection;
import adt.newportdetection.runner.ProcessWindows;
import adt.newportdetection.runner.SendInfoToMain;
import adt.repository.PortRepository;
import adt.repository.ScanInfoRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;


import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

import java.util.List;
import java.util.concurrent.Executors;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.lang.InterruptedException;
import java.lang.Thread;

import lombok.extern.slf4j.Slf4j;

@Service
@Slf4j
public class ScheduleService {

    private final MainNodeClient mainNodeClient;
    private final HostScannerService hostScannerService;
    private final PortScannerService portScannerService;
    private final SendAssetService sendAssetService;

    private final ScanInfoRepository scanInfoRepository;
    private final PortRepository portRepository;

    private final SendInfoToMain sendInfoToMain;

    private List<Vlan> vlan;
    private Node node;

    private ScheduledExecutorService scheduledExecutorService;
    private ScheduledFuture<?> futureTask;
    private Runnable task;

    private Long prevSystemDelay;
    private Long iniExecutionTime;

    private boolean newPortChecker = false;
    private ListPortDetection listPortDetection;
    private List<String[]> vlanIps;



    private class ScheduledTask implements Runnable {
        private Process process;
        private ProcessWindows processWindows;
        ThreadPoolExecutor executor_newPort;

        @Override
        public void run() {
            log.info("NEW EXECUTION SCHEDULED. Number of Vlans to scan: {}", vlan.size());
            log.info("Se ha terminado de ejecutar Scan");
            if (newPortChecker){
                process.destroy();
                executor_newPort.shutdownNow();
                log.info("Apagado del newPort");
            }
//            iniExecutionTime = System.currentTimeMillis();
//
//            CountDownLatch latch = new CountDownLatch(vlan.size());
//            ThreadPoolExecutor executor = (ThreadPoolExecutor) Executors.newFixedThreadPool(vlan.size());
//
//            // Timestamp ini
//            Date iniTimestamp_scan = new Date();
//
//            vlan.forEach(element -> executor.submit(() -> {
//
//                        log.info("Vlan address ready to scan: " + element.getNetAddress());
//
//                        hostScannerService.scanNetwork(element.getVlanName(), element.getNetAddress(), element.getMask());
//                        log.info("SCANNING OF NETWORK {} SUCCESSFULLY COMPLETED", element.getNetAddress());
//
//                        Date endTimestamp_scan = new Date();
//
//                        ScanInfo scanInfo  = ScanInfo.createScanInfo(element.getVlanName(), node.getIp(), iniTimestamp_scan, endTimestamp_scan, "scan");
//
//                        scanInfoRepository.save(scanInfo);
//
//                        latch.countDown();
//
//            }));
//
//            try {
//                latch.await();
//            } catch (InterruptedException E) {
//                // handle
//            }
//            executor.shutdown();
//
//            List<Port> ports = portScannerService.getPortList();
//
//            portRepository.saveAll(ports);
//
//            //boolean iniFlag = false;
//
//            log.info("All networks have been successfully scanned and its assets detected. Now managing this information...\n");
//            Long systemDelay = sendAssetService.sendAssets(false, vlan);//, "scan");
//
//            boolean flag = true;
//            if(systemDelay == null){
//                flag = false;
//            }else {
//                flag = !Long.toString(systemDelay).equals(Long.toString(vlan.get(0).getDelay()));
//            }
//            if( flag ){
//
//                futureTask.cancel(false);
//                prevSystemDelay = vlan.get(0).getDelay();
//
//
//                log.info("ATTENTION, a new execution delay has been set for this scanning process.\n--Previous: {} seconds.\n--New: {} seconds.\nNext iterations will use this new delay.\n", vlan.get(0).getDelay(), systemDelay);
//
//                vlan.get(0).setDelay(systemDelay); // PONERSELO A TODOS
//
//                /* At this point in the execution, it is guaranteed that the futureTask's thread has finished, so it is safely cancelled and we can schedule a new futureTask. */
//                scheduleSetUp(node);
//            }
            // Arracamos NewPortDetection
            log.info("Resuming service NewPortDetection");
            try {
                process = new ProcessBuilder("tcpdump", "-l","-nn","-i", "any", "ip").start();
            } catch (IOException e) {
                e.printStackTrace();
            }
            int n_threads = 1;
            executor_newPort = (ThreadPoolExecutor) Executors.newFixedThreadPool(n_threads);
            BufferedReader br = new BufferedReader(new InputStreamReader(process.getInputStream()));
            listPortDetection = new ListPortDetection();
            for (int i = 0; i< n_threads;i++){
                executor_newPort.submit(()-> {
                            processWindows = new ProcessWindows(br,listPortDetection,sendInfoToMain, vlanIps);
                            processWindows.start();
                        }
                );
            }
            newPortChecker = true;
            try {
                Thread.sleep(20000);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
    }

    @Autowired
    public ScheduleService(HostScannerService hostScannerService, PortRepository portRepository, PortScannerService portScannerService, SendAssetService sendAssetService, MainNodeClient mainNodeClient, ScanInfoRepository scanInfoRepository, SendInfoToMain sendInfoToMain) {
        this.mainNodeClient = mainNodeClient;
        this.hostScannerService = hostScannerService;
        this.sendAssetService = sendAssetService;
        this.portRepository = portRepository;
        this.portScannerService = portScannerService;
        this.scanInfoRepository = scanInfoRepository;
        this.sendInfoToMain = sendInfoToMain;
        scheduledExecutorService = Executors.newSingleThreadScheduledExecutor();

        task = new ScheduledTask();
    }

    void scheduleSetUp(Node node, List<String[]> vlanIps) {


        if (this.vlan == null){
            this.node = node;
            this.vlan = node.getVlan();
            this.vlanIps = vlanIps;
        }

        if(this.prevSystemDelay != null){

            Long diff = (System.currentTimeMillis() - this.iniExecutionTime) / 1000;
            if( (diff) < this.prevSystemDelay ){

                Long remaining = (this.prevSystemDelay - diff) * 1000;
                log.info("Stalling execution until a delay of " + this.prevSystemDelay + " seconds between tasks is satisfied.\n--Time already covered: " + diff + " seconds.\n--Remaining: " + (remaining/1000) + " seconds.");
                log.info("Stalling execution until a delay of {} seconds between tasks is satisfied.\n--Time already covered: {} seconds.\n--Remaining: {} seconds.",  this.prevSystemDelay, diff, (remaining/1000));

                try{
                    Thread.sleep(remaining);
                }catch(InterruptedException ie){
                    // Handled exception
                }

            }

            this.prevSystemDelay = null;
        }

        log.info("VLAN DELAY INFO: {} seconds.", vlan.get(0).getDelay());
        futureTask = scheduledExecutorService.scheduleAtFixedRate(task, 0, vlan.get(0).getDelay(), TimeUnit.SECONDS);
    }

}
