package adt.service;

import adt.client.MainNodeClient;
import adt.client.MainNodeCom;
import adt.dto.Asset;
import adt.entity.ScanInfo;
import adt.entity.Host;
import adt.entity.Vlan;
import adt.entity.Port;
import adt.repository.HostRepository;
import adt.repository.PortRepository;
import adt.repository.ScanInfoRepository;
import adt.repository.VlanRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.ArrayList;
import java.util.Date;

@Service
@Slf4j
public class SendAssetService {

    private final MainNodeClient mainNodeClient;
    private final MainNodeCom mainNodeCom;
    private final HostRepository hostRepository;
    private final PortRepository portRepository;
    private final VlanRepository vlanRepository;
    private final ScanInfoRepository scanInfoRepository;
    private final VlanService vlanService;
    private final WrittingService writtingService;

//    public void setOnStartupService(OnStartupService onStartupService) {
//        this.onStartupService = onStartupService;
//    }

//    private OnStartupService onStartupService;

    //@Value("${vlan.name}")
    //private String vlanName;

    @Value("${token}")
    private String token;

    @Value("${execution.mode}")
    private String executionMode;

    @Value("${output.path}")
    private String outputPath;

    @Value("${main.node.address}")
    private String mainNode_ip;

    @Autowired
    public SendAssetService(MainNodeCom mainNodeCom, MainNodeClient mainNodeClient, HostRepository hostRepository, PortRepository portRepository, VlanRepository vlanRepository, ScanInfoRepository scanInfoRepository, VlanService vlanService, WrittingService writtingService) {
        this.mainNodeCom = mainNodeCom;
        this.mainNodeClient = mainNodeClient;
        this.hostRepository = hostRepository;
        this.portRepository = portRepository;
        this.vlanRepository = vlanRepository;
        this.scanInfoRepository = scanInfoRepository;
        this.vlanService = vlanService;
        this.writtingService = writtingService;
    }

    Long sendAssets(boolean iniFlag, List<Vlan> vlan){//, String discoveryMethod) {

        String discoveryMethod = "scan";

        List<ScanInfo> scansToSend = new ArrayList<>();

        if(iniFlag){
            ScanInfo check = scanInfoRepository.findFirstByDiscoveryMethodOrderByEndTimestampScan(discoveryMethod);
            if(check != null) {
                scansToSend.add(check);
            }
        }else{

            List<String> vlan_names = new ArrayList<>();
            /* May re-implement later. Seems to be quite inefficient. */
            for (int i = 0; i < vlan.size(); i++) {
                vlan_names.add(vlan.get(i).getVlanName());
            }

            List<ScanInfo> check = scanInfoRepository.findFirstByDiscoveryMethodAndVlanNameInOrderByEndTimestampScan(discoveryMethod, vlan_names);//findByVlanNameIn(vlan_names);//scanInfoRepository.findAll();

            /*log.info("##################################");
            for(int i=0; i<check.size(); i++){

                log.info("___{}", check.get(i).getVlanName());
                log.info("___{}", check.get(i).getEndTimestampScan());
            }
            log.info("##################################");*/

            if(check != null) {
                scansToSend = check;
            }
        }

        Long newDelay = null;

        if(scansToSend.size() > 0 && iniFlag == false) {

            scansToSend.forEach(element -> {
                //List<Host> hosts = hostRepository.findAll();
                List<Host> hosts = hostRepository.findAllByVlanNameAndDiscoveryMethod(element.getVlanName(), discoveryMethod);

                for (int i = 0; i < hosts.size(); i++) {

                    List<Port> ports = portRepository.findAllByHostAndDiscoveryMethodTrigger(hosts.get(i), discoveryMethod);

                    //log.info("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!{}", ports.size());
                    hosts.get(i).setPorts(ports);
                }

                element.setHost(hosts);
            });
//            onStartupService.addScan(scansToSend);
            Asset asset = Asset.createAsset(scansToSend, discoveryMethod);

            if (executionMode.equals("manual")) {

                log.info("[Manual mode execution] Assets will not be sent to Main Node. Storing data in this path: {}assets_info.json", outputPath);
                writtingService.writeJSONOutput(outputPath + "assets_info.json", asset);
            } else {

                log.info("[Automatic mode execution] Assets will be sent to Main Node. Communicating with its endpoints...");
                try {
                    //newDelay = mainNodeClient.sendAssets(/*vlanName,*/ asset, token);

                    newDelay = mainNodeCom.sendAssets(asset, token);
                } catch (Exception e){//HystrixRuntimeException ex) {
                    // handle hystrix exception here
                    log.info("exception{}", e);
                    // log.warn("{}",ex);//""Unable to connect with REST API. Scanning results have been stored and will be sent when connection is restablished.");
                }

                if (newDelay != null) {

                    log.info("Assets successfully sent!\n");
                    /*portRepository.deleteAll();
                    hostRepository.deleteAll();
                    scanInfoRepository.deleteAll();*/
                    /*try {
                        portRepository.deleteByDiscoveryMethodTrigger(discoveryMethod);
                        hostRepository.deleteByDiscoveryMethod(discoveryMethod);
                        scanInfoRepository.deleteByDiscoveryMethod(discoveryMethod);
                    }catch (Exception e){
                        log.warn("_______{}",e);
                    }*/
                    log.info("The delay value received from ADT Main Node is of " + newDelay + " seconds.\n");
                }
            }

        }

        if(iniFlag){

            log.info("STARTUP PROCESS COMPLETED SUCCESFULLY. Initiating scanning process...\n\n");
        }else {

            if (newDelay != null){

                log.info("ITERATION SUCCESSFULLY COMPLETED. Preparing for next one...\n\n");
            }else{

                log.info("Communication with ADT's Main Node was not possible. Scanning results will be stored and sent when possible.\n\n");
            }
        }
        return newDelay;
    }



    public Long sendAssetsDHCP(String vlan_id, String discoveryMethod, Date time) {

        List<ScanInfo> check = null;
        try {
            check = scanInfoRepository.findAllByIniTimestampScanAndDiscoveryMethod(time, "dhcp");//findByVlanNameIn(vlan_names);//scanInfoRepository.findAll();
        }catch(Exception e){
            log.info("########{}", e);
        }

        /*for(int i=0; i<check.size(); i++){

            log.info("????????????{}", check.get(i).getIniTimestampScan());
        }*/
        //List<Host> hosts = hostRepository.findAll();
        List<Host> host = null;
        try{
        host = hostRepository.findAllByScanInfoAndDiscoveryMethod(check.get(0), "dhcp");
        }catch(Exception e){
            log.info("########{}", e);
        }

        List<Port> ports = null;
        try{
        ports = portRepository.findAllByHostAndDiscoveryMethodTrigger(host.get(0), "dhcp");
        }catch(Exception e){
        log.info("########{}", e);
        }
        host.get(0).setPorts(ports);

        List<Host> hosts = new ArrayList<>();
        hosts.add(host.get(0));

        check.get(0).setHost(hosts);

        List<ScanInfo> scansToSend = new ArrayList<>();
        scansToSend.add(check.get(0));

//        onStartupService.addDHCP(scansToSend);
        Asset asset = Asset.createAsset(scansToSend, discoveryMethod);

        Long newDelay = null;

        if (executionMode.equals("manual")) {

            log.info("[Manual mode execution] Assets will not be sent to Main Node. Storing data in this path: {}assets_info.json", outputPath);
            writtingService.writeJSONOutput(outputPath + "dhcp_assets_info.json", asset);
        } else {

            writtingService.writeJSONOutput("./dhcp_assets_info"+ time.toString() +".json", asset);

            log.info("[Automatic mode execution] Assets will be sent to Main Node. Communicating with its endpoints...");
            newDelay = mainNodeCom.sendAssets(asset, token);
            /*try {

            } catch (HystrixRuntimeException ex) {
                // handle hystrix exception here
                // log.warn("{}",ex);//""Unable to connect with REST API. Scanning results have been stored and will be sent when connection is restablished.");
            }*/

            if (newDelay != null) {

                log.info("Assets successfully sent!\n");
                /*portRepository.deleteAll();
                hostRepository.deleteAll();
                scanInfoRepository.deleteAll();

                hostRepository.deleteAllByScanInfoAndDiscoveryMethod(check.get(0), "dhcp");
                portRepository.deleteAllByHostAndDiscoveryMethodTrigger(host.get(0), "dhcp");
                scanInfoRepository.deleteAllByIniTimestampScanAndDiscoveryMethod(time, discoveryMethod);
                */
                //log.info("The delay value received from ADT Main Node is of " + newDelay + " seconds.\n");
            }
        }

        if (newDelay != null){

            //log.info("DHCP ITERATION SUCCESSFULLY COMPLETED. Preparing for next one...\n\n");
        }else{

            log.info("Communication with ADT's Main Node was not possible. DHCP Scanning results will be stored and sent when possible.\n\n");
        }

        return newDelay;
    }

}
