package adt.tcpdump.runner;

import adt.nmap.reader.NmapXmlReader;
import adt.nmap.reader.NmapXmlReaderImpl;
import lombok.extern.slf4j.Slf4j;
import org.w3c.dom.Element;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import adt.repository.*;
import java.lang.ProcessBuilder;
import java.lang.Process;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.lang.Exception;

import adt.repository.HostRepository;
import adt.entity.Host;
import adt.entity.Port;
//import adt.client.MainNodeClient;
import adt.client.MainNodeCom;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;

import adt.service.PortScannerService;
import adt.service.SendAssetService;

import java.io.*;
import java.net.*;
import java.util.*;
import adt.entity.ScanInfo;

@Slf4j
public class TCPDumpRunnerImpl implements TCPDumpRunner {

    //private final MainNodeClient mainNodeClient;
    private final MainNodeCom mainNodeCom;

    private final PortScannerService portScannerService;
    private final SendAssetService sendAssetService;

    private String token;

    private String ip;

    private String vlanName;

    private String nodeIp;

    private final PortRepository portRepository;
    private final ScanInfoRepository scanInfoRepository;
    private final HostRepository hostRepository;

    @Autowired
    public TCPDumpRunnerImpl(MainNodeCom mainNodeCom, SendAssetService sendAssetService, PortScannerService portScannerService,
                             String nodeIp, String token, String ip, String vlanName, HostRepository hostRepository,
                             PortRepository portRepository, ScanInfoRepository scanInfoRepository) {

        this.scanInfoRepository = scanInfoRepository;
        this.portRepository = portRepository;
        this.hostRepository = hostRepository;
        //this.mainNodeClient = mainNodeClient;
        this.mainNodeCom = mainNodeCom;
        this.token = token;
        this.nodeIp = nodeIp;
        this.ip = ip;
        this.vlanName = vlanName;
        this.portScannerService = portScannerService;
        this.sendAssetService = sendAssetService;
    }


    @Override
    public void sniffTraffic() {

        log.info("[DHCP Sniffing service] Initializing with network address {}", ip);

        String netName = null;
        try {
            int ctr = 0;
            String[] pieces = null;

            while(ctr < 255) {
                pieces = ip.split("\\.");
                String ipToSend = pieces[0] +"."+ pieces[1] +"."+ pieces[2] +"."+ ctr;

                netName = checkNetworkInterfaces(ipToSend);

                if(netName != null){
                    break;
                }
                ctr = ctr +1;
            }
        }catch(SocketException se){
            //handled
        }

        if(netName != null){

            log.info("[DHCP Sniffing service] Network interface with name {} was found. Now sniffing...", netName);

            String[] tcpdumpCmd = {"tcpdump", "-i", netName, "-n", "-e", "port 67 and port 68", "-v", "-l"};
            try {
                Process p = new ProcessBuilder(tcpdumpCmd).start();
                BufferedReader in = new BufferedReader(new InputStreamReader(p.getInputStream()));
                String tcpdumpOut = null;

                while ((tcpdumpOut = in.readLine()) != null) {

                    //log.info("{}",tcpdumpOut);
                    if(tcpdumpOut.contains("Client-IP") || tcpdumpOut.contains("Your-IP")) {

                        String[] clientIP_arr = tcpdumpOut.split(" ");
                        String clientIP = clientIP_arr[clientIP_arr.length - 1];

                        String mac = "";
                        String hostname = "";

                        while (tcpdumpOut != null && !tcpdumpOut.contains("Parameter-Request")) {
                            //log.info("{}",tcpdumpOut);
                            tcpdumpOut = in.readLine();

                            if (tcpdumpOut.contains("Client-Ethernet-Address")) {
                                String[] mac_arr = tcpdumpOut.split(" ");
                                mac = mac_arr[mac_arr.length - 1];
                            }
                            if (tcpdumpOut.contains("Hostname Option 12")) {
                                String[] hostname_arr = tcpdumpOut.split(" ");
                                hostname = hostname_arr[hostname_arr.length - 1];
                                hostname = hostname.substring(1, hostname.length() - 2);
                            }

                        }


                        log.info("[DHCP Sniffing service] Discovered ACK Packet for Client IP: {} \n With hostname {} and mac {}", clientIP, hostname, mac);

                        //String params = token + "&" + clientIP + "&" + mac;
                        //int alreadyPresent = mainNodeClient.checkIP(params);
                        int alreadyPresent = mainNodeCom.checkIP(token, clientIP, mac);

                        if (alreadyPresent == 0) {
                            log.info("[DHCP Sniffing service] {} has not been discovered until now. Preparing further scannign...", clientIP);
                            // New nmap to be scheduled
                            Date iniTime = new Date();

                            ScanInfo scanInfo  = ScanInfo.createScanInfo(vlanName, nodeIp, iniTime, "dhcp");

                            Host dhcp_host = Host.createHost(clientIP, hostname, mac, vlanName, iniTime, scanInfo, "dhcp");

                            portScannerService.scanHost(dhcp_host, "dhcp");

                            Date endTime = new Date();
                            scanInfo.setEndTimestampScan(endTime);
                            dhcp_host.setTimestamp_end(endTime);

                            try {
                                scanInfoRepository.save(scanInfo);

                                hostRepository.save(dhcp_host);

                                portRepository.saveAll(portScannerService.getPortListDHCP());
                                portScannerService.setPortListDHCP(new ArrayList<Port>());

                            }catch(Exception e){
                                log.info("[RUNTIME ERROR--DHCP SNIFFING SERVICE--]---{}",e);
                            }

                            sendAssetService.sendAssetsDHCP(vlanName, "dhcp", iniTime);

                        }

                    }
                }
            }catch(Exception io){
                //handled
            }

        }else{

            log.info("[DHCP Sniffing service] No network interface was found with address {}. Aborting...", ip);

        }

    }


    public String checkNetworkInterfaces(String toMatch) throws SocketException {
        Enumeration<NetworkInterface> nets = NetworkInterface.getNetworkInterfaces();
        String netName = null;
        for (NetworkInterface netint : Collections.list(nets)) {
            netName = displayInterfaceInformation(netint, toMatch);
            if (netName != null) {
                break;
                //return netName;
            }
        }

        return netName;
    }


    public String displayInterfaceInformation(NetworkInterface netint, String toMatch) throws SocketException {

        Enumeration<InetAddress> inetAddresses = netint.getInetAddresses();
        String matched = null;
        for (InetAddress inetAddress : Collections.list(inetAddresses)) {

            if(inetAddress.getHostAddress().equals(toMatch)){

                matched = netint.getName();

                /*byte[] mac = netint.getHardwareAddress();
                String macToStore = "";

                if(mac != null) {

                    for (int i = 0; i < mac.length; i++) {

                        String toCheck = Integer.toHexString(mac[i]);
                        if(toCheck.length() > 2){

                            toCheck = toCheck.substring(toCheck.length()-3, toCheck.length()-1);
                        }

                        if(i < mac.length-1){
                            macToStore = macToStore + toCheck + ":";
                        }else{
                            macToStore = macToStore + toCheck;
                        }
                    }

                }
                */

                break;
            }
        }

        return matched;
    }



}
